function [ data ] = importInlininoNMEA( filename, verbose )
% importInlininoNMEA Import NMEA data from csv files
% Author: Guillaume Bourdin
% Date: August 1st, 2022
%
% Input:
%   - filename: <char> filename including full path
%   - verbose (optional)
% 
% Example: [ data, lambda] = importInlininoNMEA( filename, verbose )
%%
  if nargin < 2; verbose = false; end
  if verbose
    foo = strsplit(filename, '/');
    fprintf('Importing %s ... ', foo{end});
  end
  
  % % Get date from filename
  % s = strsplit(filename, '_');
  % dt_ref = s{end-1};
  
  % Open file
  fid=fopen(filename);
  if fid==-1
    error('Unable to open file: %s', filename);
  end
  
  % Get header
  hd = strip(strsplit(fgetl(fid), ','));
  if endsWith(filename, '.log')
    raw_format = true;
  elseif strcmp(hd{2}, 'packet')
    raw_format = true;
  else
    raw_format = false;
  end
  
  if raw_format
    t = readlines(filename);
    % parse RMC sentences: Recommended Minimum Navigation Information
    t_rmc = t(contains(t,'RMC,'));
    data = parseRMC(t_rmc);
    % parse MWD sentences: True Wind
    t_mwd = t(contains(t,'MWD,'));
    if ~isempty(t_mwd)
      data_mwd = parseMWD(t_mwd);
      data_mwd = round_timestamp(data_mwd, seconds(0.001));
      % merge mwd
      data.true_wind_dir_mwd = interp1(data_mwd.dt, data_mwd.true_wind_dir, data.dt, 'nearest');
      data.true_wind_spd_mwd = interp1(data_mwd.dt, data_mwd.true_wind_spd, data.dt, 'nearest');
    else
      data.true_wind_dir_mwd = NaN(size(data.dt));
      data.true_wind_spd_mwd = NaN(size(data.dt));
    end
    % parse MWV sentences: Apparent Wind
    t_mwv = t(contains(t,'MWV,'));
    if ~isempty(t_mwv)
      data_mwv = parseMWV(t_mwv);
      data_mwv = round_timestamp(data_mwv, seconds(0.001));
      % merge mwv
      data.true_wind_dir_mwv = interp1(data_mwv.dt, data_mwv.true_wind_dir, data.dt, 'nearest');
      data.true_wind_spd_mwv = interp1(data_mwv.dt, data_mwv.true_wind_spd, data.dt, 'nearest');
      data.apparent_wind_dir = interp1(data_mwv.dt, data_mwv.apparent_wind_dir, data.dt, 'nearest');
      data.apparent_wind_spd = interp1(data_mwv.dt, data_mwv.apparent_wind_spd, data.dt, 'nearest');
    else
      data.true_wind_dir_mwv = NaN(size(data.dt));
      data.true_wind_spd_mwv = NaN(size(data.dt));
      data.apparent_wind_dir = NaN(size(data.dt));
      data.apparent_wind_spd = NaN(size(data.dt));
    end
    % parse MMB sentences: Atmospheric Pressure
    t_mmb = t(contains(t,'MMB,'));
    if ~isempty(t_mmb)
      data_mmb = parseMMB(t_mmb);
      data_mmb = round_timestamp(data_mmb, seconds(0.001));
      % merge mmb
      data.atm_press = interp1(data_mmb.dt, data_mmb.atm_press, data.dt, 'nearest');
    else
      data.atm_press = NaN(size(data.dt));
    end
    % parse VPW sentences: Speed - Measured Parallel to Wind
    t_vpw = t(contains(t,'VPW,'));
    if ~isempty(t_vpw)
      data_vpw = parseVPW(t_vpw);
      data_vpw = round_timestamp(data_vpw, seconds(0.001));
      % merge vpw
      data.spd_wind_relative = interp1(data_vpw.dt, data_vpw.spd_wind_relative, data.dt, 'nearest');
    else
      data.spd_wind_relative = NaN(size(data.dt));
    end
    % parse VHW sentences: Water speed and heading
    t_vhw = t(contains(t,'VHW,'));
    if ~isempty(t_vhw)
      data_vhw = parseVHW(t_vhw);
      data_vhw = round_timestamp(data_vhw, seconds(0.001));
      % merge vhw
      data.heading_true_vhw = interp1(data_vhw.dt, data_vhw.heading_true, data.dt, 'nearest');
      data.heading_magnetic_vhw = interp1(data_vhw.dt, data_vhw.heading_magnetic, data.dt, 'nearest');
      data.spd_water_relative = interp1(data_vhw.dt, data_vhw.spd_water_relative, data.dt, 'nearest');
    else
      data.heading_true_vhw = NaN(size(data.dt));
      data.heading_magnetic_vhw = NaN(size(data.dt));
      data.spd_water_relative = NaN(size(data.dt));
    end
    % parse DBT sentences: Depth below transducer
    t_dbt = t(contains(t,'DBT,'));
    if ~isempty(t_dbt)
      data_dbt = parseDBT(t_dbt);
      data_dbt = round_timestamp(data_dbt, seconds(0.001));
      % merge dbt
      data.depth_m = interp1(data_dbt.dt, data_dbt.depth_m, data.dt, 'nearest');
    else
      data.depth_m = NaN(size(data.dt));
    end
    % parse MTW sentences: Mean Temperature of Water
    t_mtw = t(contains(t,'MTW,'));
    if ~isempty(t_mtw)
      data_mtw = parseMTW(t_mtw);
      data_mtw = round_timestamp(data_mtw, seconds(0.001));
      % merge mtw
      data.water_temp = interp1(data_mtw.dt, data_mtw.water_temp, data.dt, 'nearest');
    else
      data.water_temp = NaN(size(data.dt));
    end
    % parse MTA sentences: Air Temperature
    t_mta = t(contains(t,'MTA,'));
    if ~isempty(t_mta)
      data_mta = parseMTA(t_mta);
      data_mta = round_timestamp(data_mta, seconds(0.001));
      % merge mta
      data.air_temp = interp1(data_mta.dt, data_mta.air_temp, data.dt, 'nearest');
    else
      data.air_temp = NaN(size(data.dt));
    end
    % remove empty rows
    data(all(isnat(data.dt) & isnat(data.gps_dt) & isnan(data.lat) & isnan(data.lon), 2), :) = [];
    % data.dt = datenum(data.dt);
    % data.gps_dt = datenum(data.gps_dt);
  else
    hd{strcmp(hd, 'time')} = 'dt';
    hd{strcmp(hd, 'datetime')} = 'gps_dt';
    hd{strcmp(hd, 'latitude')} = 'lat';
    hd{strcmp(hd, 'longitude')} = 'lon';
    % get units skipping empty lines (bug in old Inlinino)
    unit = fgetl(fid);
    while isempty(unit)
      unit = fgetl(fid);
    end
    % get units and lambda
    unit = strip(strsplit(strrep(unit, ',,', ', ,'), ','));
  
    % Set parser
    parser = ['%s%s' repmat('%f', 1, size(hd,2)-2)];
  
    % Read data
    t = textscan(fid, parser, 'delimiter',',');
    % Close file
    fclose(fid);
  
    % remove GPS time
    unit = unit(~strcmp(hd, 'gps_dt'));
    t = t(~strcmp(hd, 'gps_dt'));
    hd = hd(~strcmp(hd, 'gps_dt'));
  
    % Build table starting by datetime
    id_dt = strcmp(hd, 'dt');
    if any(id_dt)
      if contains(t{id_dt}(1), '/')
        dt = datetime(t{id_dt}, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS');
        % dt = datenum(t{i}, 'yyyy/mm/dd HH:MM:SS.FFF');
      else
        dt = datetime(cellfun(@(x) [dt_ref x], t{id_dt}, 'UniformOutput', false), 'InputFormat', 'yyyyMMddHH:mm:ss.SSS');
        % dt = datenum(cellfun(@(x) [dt_ref x], t{1}, 'UniformOutput', false), 'yyyymmddHH:MM:SS.FFF');
      end
    else
      error('No datetime variable found in file')
    end
    % extract all other variables
    t = t(~strcmp(hd, 'dt'));
    hd = hd(~strcmp(hd, 'dt'));
    dat = [];
    for i = 1:size(hd, 2)
      if size(dat,1) ~= size(t{i},1) && size(dat,1) > 0
        warning('Missing data on last %i row of file: deleted\n', size(dat,1)-size(t{i},1))
        dt = dt(1:size(t{i}, 1), :);
        dat = dat(1:size(t{i}, 1), :);
      end
      % if contains(hd{i}, 'swt')
      %   foo2 = t{i};
      %   foo2(contains(foo2, 'True')) = {'1'};
      %   foo2(contains(foo2, 'False')) = {'0'};
      %   dat = [dat logical(str2num(cell2mat(foo2)))];
      % else
      dat = [dat t{i}];
      % end
    end


    % % Build table
    % dat = [];
    % for i = 1:size(hd, 2)
    %   if size(dat,1) ~= size(t{i},1) && size(dat,1) > 0
    %     warning('Missing data on last %i row of file: deleted\n', size(dat,1)-size(t{i},1))
    %     dat = dat(1:size(t{i}, 1), :);
    %   end
    %   if strcmp(hd{i}, 'dt')
    %     % dat = [dat datenum(t{i}, 'yyyy/mm/dd HH:MM:SS.FFF')];
    %     dat = [dat datetime(t{i}, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS')];
    %   elseif contains(hd{i}, 'swt')
    %     foo2 = t{i};
    %     foo2(contains(foo2, 'True')) = {'1'};
    %     foo2(contains(foo2, 'False')) = {'0'};
    %     dat = [dat logical(cell2mat(foo2))];
    %   else
    %     dat = [dat t{i}];
    %   end
    % end
    data = array2table(dat, 'VariableNames', hd);
    data = addvars(data, dt, 'NewVariableNames', 'dt', 'Before', hd{1});
    data.Properties.VariableUnits = unit;
  end
  
  % Remove last line if it's past midnight (bug in old Inlinino)
  if ~isempty(data) && size(data,1) > 1
    if data.dt(end-1) > data.dt(end)
      data(end,:) = [];
    end
  end
  
  if verbose; fprintf('Done\n'); end
end

function data = parseRMC(t)
  data = cell2table(cellstr(split(t, {',', '*'})), 'VariableNames', ...
    {'dt','msg_ID','time','status','lat','lat_SN','lon','lon_WE','sog','cog',...
    'date','magnetic_var','magnetic_var_WE','nav_status','check_sum'});
  data(cellfun('isempty', data.lat) | cellfun('isempty', data.lon), :) = [];
  data.dt = datetime(data.dt, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS');
  data = addvars(data, datetime(strcat(data.date, data.time), 'InputFormat', 'ddMMyyHHmmss.SS'), ...
    'NewVariableNames', 'gps_dt', 'After','dt');
  data.lat = str2double(cellfun(@(c) c(1:2), data.lat, 'un', 0)) + str2double(cellfun(@(c) c(3:end), data.lat, 'un', 0))/100;
  data.lat(strcmp(data.lat_SN, 'S')) = -data.lat(strcmp(data.lat_SN, 'S'));
  data.lon = str2double(cellfun(@(c) c(1:3), data.lon, 'un', 0)) + str2double(cellfun(@(c) c(4:end), data.lon, 'un', 0))/100;
  data.lon(strcmp(data.lon_WE, 'W')) = -data.lon(strcmp(data.lon_WE, 'W'));
  data.sog = str2double(data.sog);
  data.cog = str2double(data.cog);
  data = addvars(data, deg2rad(data.cog), 'NewVariableNames', 'magnetic_cog', 'After','cog');
  magnetic_var = deg2rad(str2double(data.magnetic_var));
  data.magnetic_cog(strcmp(data.magnetic_var_WE, 'E')) = data.magnetic_cog(strcmp(data.magnetic_var_WE, 'E')) - magnetic_var(strcmp(data.magnetic_var_WE, 'E'));
  data.magnetic_cog(strcmp(data.magnetic_var_WE, 'W')) = data.magnetic_cog(strcmp(data.magnetic_var_WE, 'W')) + magnetic_var(strcmp(data.magnetic_var_WE, 'W'));
  data.magnetic_cog = rad2deg(data.magnetic_cog);
  data = removevars(data, {'msg_ID','status','lat_SN','lon_WE','date','time',...
    'magnetic_var' ,'magnetic_var_WE','nav_status','check_sum'});
end

function data = parseMWD(t)
  data = cell2table(cellstr(split(t, {',', '*'})), 'VariableNames', ...
    {'dt','msg_ID','wind_dir1','ref1','wind_dir2','ref2','wind_spd1','wind_spd1_unit','wind_spd2','wind_spd2_unit','check_sum'});
  data.dt = datetime(data.dt, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS');
  data.wind_dir1 = str2double(data.wind_dir1);
  data.wind_dir2 = str2double(data.wind_dir2);
  data.wind_spd1 = str2double(data.wind_spd1);
  data.wind_spd2 = str2double(data.wind_spd2);
  data.true_wind_dir = NaN(size(data.dt));
  data.true_wind_dir(strcmp(data.ref1, 'T')) = data.wind_dir1(strcmp(data.ref1, 'T'));
  data.true_wind_dir(strcmp(data.ref2, 'T')) = data.wind_dir2(strcmp(data.ref2, 'T'));
  data.true_wind_spd = NaN(size(data.dt));
  data.true_wind_spd(strcmp(data.wind_spd1_unit, 'N')) = data.wind_spd1(strcmp(data.wind_spd1_unit, 'N'));
  data.true_wind_spd(strcmp(data.wind_spd2_unit, 'N')) = data.wind_spd2(strcmp(data.wind_spd2_unit, 'N'));
  data = removevars(data, {'msg_ID','wind_dir1','wind_dir2','wind_spd1',...
    'wind_spd1_unit','wind_spd2','wind_spd2_unit','ref1','ref2','check_sum'});
end

function data = parseMWV(t)
  data = cell2table(cellstr(split(t, {',', '*'})), 'VariableNames', ...
    {'dt','msg_ID','wind_angle','ref','wind_spd','wind_spd_unit','status','check_sum'});
  data.dt = datetime(data.dt, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS');
  data.wind_angle = str2double(data.wind_angle);
  data.wind_spd = str2double(data.wind_spd);
  % check if ref == T, get true wind data
  data.true_wind_dir = NaN(size(data.dt));
  data.true_wind_spd = NaN(size(data.dt));
  data.true_wind_dir(strcmp(data.ref, 'T')) = data.wind_angle(strcmp(data.ref, 'T'));
  data.true_wind_spd(strcmp(data.ref, 'T')) = data.wind_spd(strcmp(data.ref, 'T'));
  % check if ref == R, get apparent wind data
  data.apparent_wind_dir = NaN(size(data.dt));
  data.apparent_wind_spd = NaN(size(data.dt));
  data.apparent_wind_dir(strcmp(data.ref, 'R')) = data.wind_angle(strcmp(data.ref, 'R'));
  data.apparent_wind_spd(strcmp(data.ref, 'R')) = data.wind_spd(strcmp(data.ref, 'R'));
  data = removevars(data, {'msg_ID','wind_angle','ref','wind_spd','wind_spd_unit','status','check_sum'});
end

function data = parseMMB(t)
  data = cell2table(cellstr(split(t, {',', '*'})), 'VariableNames', ...
    {'dt','msg_ID','atm_press1','atm_press_unit1','atm_press2','atm_press_unit2','check_sum'});
  data.dt = datetime(data.dt, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS');
  data.atm_press1 = str2double(data.atm_press1);
  data.atm_press2 = str2double(data.atm_press2);
  % get atm pressure in bar
  data.atm_press = NaN(size(data.dt));
  data.atm_press(strcmp(data.atm_press_unit1, 'B')) = data.atm_press1(strcmp(data.atm_press_unit1, 'B'));
  data.atm_press(strcmp(data.atm_press_unit2, 'B')) = data.atm_press2(strcmp(data.atm_press_unit2, 'B'));
  data = removevars(data, {'msg_ID','atm_press1','atm_press_unit1','atm_press2','atm_press_unit2','check_sum'});
end

function data = parseVPW(t)
  data = cell2table(cellstr(split(t, {',', '*'})), 'VariableNames', ...
    {'dt','msg_ID','spd_wind_relative1','spd_unit1','spd_wind_relative2','spd_unit2','check_sum'});
  data.dt = datetime(data.dt, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS');
  data.spd_wind_relative1 = str2double(data.spd_wind_relative1);
  data.spd_wind_relative2 = str2double(data.spd_wind_relative2);
  % get speed - measured parallel to wind in knots
  data.spd_wind_relative = NaN(size(data.dt));
  data.spd_wind_relative(strcmp(data.spd_unit1, 'N')) = data.spd_wind_relative1(strcmp(data.spd_unit1, 'N'));
  data.spd_wind_relative(strcmp(data.spd_unit2, 'N')) = data.spd_wind_relative2(strcmp(data.spd_unit2, 'N'));
  data = removevars(data, {'msg_ID','spd_wind_relative1','spd_unit1','spd_wind_relative2','spd_unit2','check_sum'});
end

function data = parseVHW(t)
  data = cell2table(cellstr(split(t, {',', '*'})), 'VariableNames', ...
    {'dt','msg_ID','heading_true','ref_T','heading_magnetic','ref_M','spd_water_relative1',...
    'spd_unit1','spd_water_relative2','spd_unit2','check_sum'});
  data.dt = datetime(data.dt, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS');
  data.heading_true = str2double(data.heading_true);
  data.heading_magnetic = str2double(data.heading_magnetic);
  data.spd_water_relative1 = str2double(data.spd_water_relative1);
  data.spd_water_relative2 = str2double(data.spd_water_relative2);
  % get speed - measured parallel to wind in knots
  data.spd_water_relative = NaN(size(data.dt));
  data.spd_water_relative(strcmp(data.spd_unit1, 'N')) = data.spd_water_relative1(strcmp(data.spd_unit1, 'N'));
  data.spd_water_relative(strcmp(data.spd_unit2, 'N')) = data.spd_water_relative2(strcmp(data.spd_unit2, 'N'));
  data = removevars(data, {'msg_ID','ref_T','ref_M','spd_water_relative1','spd_unit1','spd_water_relative2','spd_unit2','check_sum'});
end

function data = parseDBT(t)
  data = cell2table(cellstr(split(t, {',', '*'})), 'VariableNames', ...
    {'dt','msg_ID','depth_f','unit_f','depth_m','unit_m','depth_F','unit_F','check_sum'});
  data.dt = datetime(data.dt, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS');
  data.depth_m = str2double(data.depth_m);
  data = removevars(data, {'msg_ID','depth_f','unit_f','unit_m','depth_F','unit_F','check_sum'});
end

function data = parseMTW(t)
  data = cell2table(cellstr(split(t, {',', '*'})), 'VariableNames', ...
    {'dt','msg_ID','water_temp','unit','check_sum'});
  data.dt = datetime(data.dt, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS');
  data.water_temp = str2double(data.water_temp);
  data = removevars(data, {'msg_ID','unit','check_sum'});
end

function data = parseMTA(t)
  data = cell2table(cellstr(split(t, {',', '*'})), 'VariableNames', ...
    {'dt','msg_ID','air_temp','unit','check_sum'});
  data.dt = datetime(data.dt, 'InputFormat', 'yyyy/MM/dd HH:mm:ss.SSS');
  data.air_temp = str2double(data.air_temp);
  data = removevars(data, {'msg_ID','unit','check_sum'});
end

